/*
 *  (C) Copyright 8 X 2004 Michal 'GiM' Spadlinski <gim at skrzynka dot pl>
 *
 *  This is simple pipe() command presentation
 *  + creates pipe() and fork()
 *  + redirects using dup2():
 *    - stdout in child to one end of pipe
 *    - stdin in parent to the other end of pipe
 *  + installs new signal 10 handler in child and wait for signal with pause()
 *  + installs dummy signal SIGLARM handler and waits 3 seconds
 *  + sends signal to child
 *  + read data from pipe [content of /etc/passwd]
 *  + wait for dead child
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* 
 * Compilation: bash$ gcc -o g_fifo{,.c} -Wall
 */
#include <stdio.h>
#include <signal.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

void sig_from_parent(int sig)
{
	int fd, readed, len;
	struct stat st;
	char buf[4096];

	if ((fd = open("/etc/passwd", O_RDONLY)) == -1)
	{
		perror("Couldn't open /etc/passwd");
		return;
	}
	if (fstat(fd, &st) == -1) 
		len = 4096;
	else {
		len = st.st_blksize>4096 ? 4096:st.st_blksize;
	}
	while ((readed = read(fd, buf, len)))
		write(1, buf, readed);

	close(fd);
	return;
	
}

void dummy_ignore(int sig)
{
	return;
}

int main(int argc, char **argv, char **envp)
{
	int cpid, st, fd[2], len;
	char bufor[4096];

	if (pipe(fd) == -1)
	{
		perror("pipe() failed");
		return -1;
	}
	if ((cpid = fork()) == -1)
	{
		perror("fork() failed");
		return -1;
	}
	
	/* child */
	if (!cpid)
	{
		printf("CPID: %d\n", getpid());
		close(fd[0]);
		dup2(fd[1], 1);
		close(fd[1]);
		signal(10, sig_from_parent);
		
		pause();

		return 0;
	/* parent */
	} else {
		printf("PPID: %d\n", getpid());
		close(fd[1]);
		dup2(fd[0], 0);
		close(fd[0]);

		signal(14, dummy_ignore);
		alarm(3);
		pause();

		printf("3 seconds passed... sending signal...\n");
		kill(cpid, 10);
		printf("Reading data from child...\n");
		while ((len=read(0, bufor, 4096)))
			printf(bufor);
				
		if (waitpid(cpid, &st, 0) == -1)
			perror("waitpid()");
	}

	return 0;
}
